/* version 1.2 */
#ifndef _MCF5282_H_
#define _MCF5282_H_

/* custom data types */
typedef unsigned long   lword;
typedef unsigned short	word;
typedef unsigned char	byte;

/* function returning the clock frequency in Hz */
unsigned clock_freq();


/* register memory map */

#define ISPBAR		(0x40000000)


/* UART 0 */
#define UART_UMR10	((volatile byte*)(ISPBAR+0x200))	/* UART mode registers                                                   */
#define UART_USR0	((volatile byte*)(ISPBAR+0x204))	/* UART status registers     / clock select                              */
#define UART_UCR0	((volatile byte*)(ISPBAR+0x208))	/* 	-	       	     / UART command registers                    */
#define UART_URB0	((volatile byte*)(ISPBAR+0x20C))	/* UART receiver buffer      / transmitter buffer                        */
#define UART_UTB0	((volatile byte*)(ISPBAR+0x20C))	/* UART receiver buffer      / transmitter buffer                        */
#define UART_UIOCR0	((volatile byte*)(ISPBAR+0x210))	/* UART input port change    / auxiliary control registers               */
#define UART_UACR0	((volatile byte*)(ISPBAR+0x210))	/* UART input port change    / auxiliary control registers               */
#define UART_UIMR0	((volatile byte*)(ISPBAR+0x214))	/* UART interrupt register   / interrupt mask registers                  */
#define UART_UBG10	((volatile byte*)(ISPBAR+0x218))	/* UART 	-	     / divider upper registers                   */
#define UART_UBG20	((volatile byte*)(ISPBAR+0x21C))	/* UART 	-	     / divider lower registers                   */
#define UART_UIP0	((volatile byte*)(ISPBAR+0x234))	/* UART input port registers / 		-                                */
#define UART_UOP10	((volatile byte*)(ISPBAR+0x238))	/* UART 	-	     / output port bit set command registers 1   */
#define UART_UOP00	((volatile byte*)(ISPBAR+0x23C))	/* UART 	-	     / output port bit set command registers 0   */

#define USR_RB		0x80			/* received break */
#define USR_FE		0x40			/* framing error */
#define USR_PE		0x20			/* parity error */
#define USR_OE		0x10			/* overrun error */
#define USR_TxEMP	0x08			/* transmitter empty */
#define USR_TxRDY	0x04			/* transmitter ready */
#define USR_FFULL	0x02			/* fifo full */
#define USR_RxRDY	0x01			/* receiver ready */

#define UMR1_FFULL	0x40			/* fifo full */

#define UIMR_FFULL	0x2			/* fifo full */

#define SYNCR	((volatile word*) (ISPBAR+0x120000))	/* clock synthesizer control */
/* SYNCR bit masks */
#define SYNCR_LOLRE     0x8000  /* loss of lock reset enable */
#define SYNCR_MFD	0x7000	/* multiplication factor divider*/
#define SYNCR_LOCRE     0x0800  /* loss of clock reset enable */
#define SYNCR_RFD	0x0700	/* reduced frequency divider  */
#define SYNCR_LOCEN	0x0080	/* enable the loss of clock   */
#define SYNCR_DISCLK	0x0040  /* disable clock out */
#define SYNCR_FWKUP     0x0020  /* fast wakeup */
#define SYNCR_STPMD     0x000c  /* PLL and CLKOUT operation in stop mode */ 

/* interrupts */
#define INTC0	(ISPBAR + 0xc00)
#define INTC1   (ISPBAR + 0xd00)
#define GIACK   (ISPBAR + 0xf00)

#define IPRH0	((volatile lword*)(INTC0 + 0x00)) /* interrupt pending register high */
#define IPRL0	((volatile lword*)(INTC0 + 0x04)) /* interrupt pending register low */
#define IMRH0	((volatile lword*)(INTC0 + 0x08)) /* interrupt mask register high */
#define IMRL0	((volatile lword*)(INTC0 + 0x0C)) /* interrupt mask register low */
#define INTFRCH0	((volatile lword*)(INTC0 + 0x10)) /* interrupt force register high */
#define INTFRCL0	((volatile lword*)(INTC0 + 0x14)) /* interrupt force register low */
#define IRLR0	((volatile byte*)(INTC0 + 0x18)) /* interrupt request level register */
#define IACKLPR0	((volatile byte*)(INTC0 + 0x19)) /* interrupt ack level and priority register) */
#define ICR0 ((volatile byte*)(INTC0 + 0x40)) /* interrupt control registers */
#define SWIACKR0	((volatile byte*)(INTC0 + 0xe0)) /* software IACK register */
#define L1IACK10	((volatile byte*)(INTC0 + 0xe4)) /* level 1 IACK register */
#define L1IACK20	((volatile byte*)(INTC0 + 0xe8)) /* level 2 IACK register */
#define L1IACK30	((volatile byte*)(INTC0 + 0xec)) /* level 3 IACK register */
#define L1IACK40	((volatile byte*)(INTC0 + 0xf0)) /* level 4 IACK register */
#define L1IACK50	((volatile byte*)(INTC0 + 0xf4)) /* level 5 IACK register */
#define L1IACK60	((volatile byte*)(INTC0 + 0xf8)) /* level 6 IACK register */
#define L1IACK70	((volatile byte*)(INTC0 + 0xfc)) /* level 7 IACK register */

/* port registers */
/* irg1-irq7 registers */
#define EPPAR		((volatile word*)(INTC0 + 0x130000)) /* Port assignment register */
#define EPDDR		((volatile byte*)(INTC0 + 0x130002)) /* Data direction register */
#define EPIER		((volatile byte*)(INTC0 + 0x130003)) /* Port interrupt enable register */
#define EPDR		((volatile byte*)(INTC0 + 0x130004)) /* Port data register */
#define EPPDR		((volatile byte*)(INTC0 + 0x130005)) /* Edge port pin data register */
#define EPFR		((volatile byte*)(INTC0 + 0x130006)) /* Port flag register  register*/

/* Port output data registers */
#define PORTA		((volatile byte*)(INTC0 + 0x100000)) /* Port A output data  register */
#define PORTB		((volatile byte*)(INTC0 + 0x100001)) /* Port B output data  register */
#define PORTC		((volatile byte*)(INTC0 + 0x100002)) /* Port C output data  register */
#define PORTD		((volatile byte*)(INTC0 + 0x100003)) /* Port D output data  register */
#define PORTE		((volatile byte*)(INTC0 + 0x100004)) /* Port E output data  register */
#define PORTF		((volatile byte*)(INTC0 + 0x100005)) /* Port F output data  register */
#define PORTG		((volatile byte*)(INTC0 + 0x100006)) /* Port G output data  register */
#define PORTH		((volatile byte*)(INTC0 + 0x100007)) /* Port H output data  register */
#define PORTJ		((volatile byte*)(INTC0 + 0x100008)) /* Port J output data  register */
#define PORTDD		((volatile byte*)(INTC0 + 0x100009)) /* Port DD output data  register */
#define PORTEH		((volatile byte*)(INTC0 + 0x10000A)) /* Port EH output data  register */
#define PORTEL		((volatile byte*)(INTC0 + 0x10000B)) /* Port EL output data  register */
#define PORTQS		((volatile byte*)(INTC0 + 0x10000D)) /* Port QS output data  register */
#define PORTAS		((volatile byte*)(INTC0 + 0x10000C)) /* Port AS output data  register */
#define PORTSD		((volatile byte*)(INTC0 + 0x10000E)) /* Port SD output data  register */
#define PORTTC		((volatile byte*)(INTC0 + 0x10000F)) /* Port TC output data  register */
#define PORTTD		((volatile byte*)(INTC0 + 0x100010)) /* Port TD output data  register */
#define PORTUA		((volatile byte*)(INTC0 + 0x100011)) /* Port UA output data  register */

/* port data direction registers */
#define DDRA		((volatile byte*)(INTC0 + 0x100014)) /* Port A data direction register */
#define DDRB		((volatile byte*)(INTC0 + 0x100015)) /* Port B data direction register */
#define DDRC		((volatile byte*)(INTC0 + 0x100016)) /* Port C data direction register */
#define DDRD		((volatile byte*)(INTC0 + 0x100017)) /* Port D data direction register */
#define DDRE		((volatile byte*)(INTC0 + 0x100018)) /* Port E data direction register */
#define DDRF		((volatile byte*)(INTC0 + 0x100019)) /* Port F data direction register */
#define DDRG		((volatile byte*)(INTC0 + 0x10001A)) /* Port G data direction register */
#define DDRH		((volatile byte*)(INTC0 + 0x10001B)) /* Port H data direction register */
#define DDRJ		((volatile byte*)(INTC0 + 0x10001C)) /* Port J data direction register */
#define DDRDD		((volatile byte*)(INTC0 + 0x10001D)) /* Port DD data direction register */
#define DDREH		((volatile byte*)(INTC0 + 0x10001E)) /* Port EH data direction register */
#define DDREL		((volatile byte*)(INTC0 + 0x10001F)) /* Port EL data direction register */
#define DDRQS		((volatile byte*)(INTC0 + 0x100021)) /* Port QS data direction register */
#define DDRAS		((volatile byte*)(INTC0 + 0x100020)) /* Port AS data direction register */
#define DDRSD		((volatile byte*)(INTC0 + 0x100022)) /* Port SD data direction register */
#define DDRTC		((volatile byte*)(INTC0 + 0x100023)) /* Port TC data direction register */
#define DDRTD		((volatile byte*)(INTC0 + 0x100024)) /* Port RD data direction register */
#define DDRUA		((volatile byte*)(INTC0 + 0x100025)) /* Port UA data direction register */

/* port pin data/set data registers */
#define PORTAP 		((volatile byte*)(INTC0 + 0x100028)) /* Port pin A data/set data registers */
#define PORTBP 		((volatile byte*)(INTC0 + 0x100029)) /* Port pin B data/set data registers */
#define PORTCP 		((volatile byte*)(INTC0 + 0x10002A)) /* Port pin C data/set data registers */
#define PORTDP 		((volatile byte*)(INTC0 + 0x10002B)) /* Port pin D data/set data registers */
#define PORTEP 		((volatile byte*)(INTC0 + 0x10002C)) /* Port pin E data/set data registers */
#define PORTFP 		((volatile byte*)(INTC0 + 0x10002D)) /* Port pin F data/set data registers */
#define PORTGP 		((volatile byte*)(INTC0 + 0x10002E)) /* Port pin G data/set data registers */
#define PORTHP 		((volatile byte*)(INTC0 + 0x10002F)) /* Port pin H data/set data registers */
#define PORTJP 		((volatile byte*)(INTC0 + 0x100030)) /* Port pin J data/set data registers */
#define PORTDDP 	((volatile byte*)(INTC0 + 0x100031)) /* Port pin DD data/set data registers */
#define PORTEHP		((volatile byte*)(INTC0 + 0x100032)) /* Port pin EHP data/set data registers */
#define PORTELP		((volatile byte*)(INTC0 + 0x100033)) /* Port pin EHL data/set data registers */
#define PORTQSP		((volatile byte*)(INTC0 + 0x100035)) /* Port pin QS data/set data registers */
#define PORTASP		((volatile byte*)(INTC0 + 0x100034)) /* Port pin AS data/set data registers */
#define PORTSDP		((volatile byte*)(INTC0 + 0x100036)) /* Port pin SD data/set data registers */

/* port clear out data registers */
#define CLRA		((volatile byte*)(INTC0 + 0x10003C)) /* Port A clear data register*/
#define CLRB		((volatile byte*)(INTC0 + 0x10003D)) /* Port B clear data register*/
#define CLRC		((volatile byte*)(INTC0 + 0x10003E)) /* Port C clear data register*/
#define CLRD		((volatile byte*)(INTC0 + 0x10003F)) /* Port D clear data register*/
#define CLRE		((volatile byte*)(INTC0 + 0x100040)) /* Port E clear data register*/
#define CLRF		((volatile byte*)(INTC0 + 0x100041)) /* Port F clear data register*/
#define CLRG		((volatile byte*)(INTC0 + 0x100042)) /* Port G clear data register*/
#define CLRH		((volatile byte*)(INTC0 + 0x100043)) /* Port H clear data register*/
#define CLRJ		((volatile byte*)(INTC0 + 0x100044)) /* Port J clear data register*/
#define CLRDD		((volatile byte*)(INTC0 + 0x100045)) /* Port DD clear data register*/
#define CLREH		((volatile byte*)(INTC0 + 0x100046)) /* Port EH clear data register*/
#define CLREL		((volatile byte*)(INTC0 + 0x100047)) /* Port EL clear data register*/
#define CLRQS		((volatile byte*)(INTC0 + 0x100049)) /* Port QS clear data register*/
#define CLRAS		((volatile byte*)(INTC0 + 0x100048)) /* Port AS clear data register*/
#define CLRSD		((volatile byte*)(INTC0 + 0x10004A)) /* Port SD clear data register*/
#define CLRTC		((volatile byte*)(INTC0 + 0x10004B)) /* Port TC clear data register*/
#define CLRTD		((volatile byte*)(INTC0 + 0x10004C)) /* Port TD clear data register*/
#define CLRUA		((volatile byte*)(INTC0 + 0x10004D)) /* Port UA clear data register*/

/* ports assignment registers */
#define PBCDPAR		((volatile byte*)(INTC0 + 0x100050)) /* Port B,C,D pin assigment register */
#define PEPAR		((volatile word*)(INTC0 + 0x100052)) /* Port E pin assigment register */
#define PFPAR		((volatile byte*)(INTC0 + 0x100051)) /* Port F pin assigment register */
#define PJPAR		((volatile byte*)(INTC0 + 0x100054)) /* Port J pin assigment register */
#define PSDPAR		((volatile byte*)(INTC0 + 0x100055)) /* Port SD pin assigment register */
#define PASPAR		((volatile word*)(INTC0 + 0x100056)) /* Port AS pin assigment register */
#define PEHLPAR		((volatile byte*)(INTC0 + 0x100058)) /* Port EH,EL pin assigment register */
#define PQSPAR		((volatile byte*)(INTC0 + 0x100059)) /* Port QS pin assigment register */
#define PTCPAR		((volatile byte*)(INTC0 + 0x10005A)) /* Port TC pin assigment register */
#define PTDPAR		((volatile byte*)(INTC0 + 0x10005B)) /* Port TD pin assigment register */
#define PUAPAR		((volatile byte*)(INTC0 + 0x10005C)) /* Port UA pin assigment register */


#define UART0_INTVECTOR	13

#define PCSR0	((volatile word*)(ISPBAR + 0x150000)) /* PIT control and status register  */
#define PMR0	((volatile word*)(ISPBAR + 0x150002)) /* PIT modulus register  */
#define PCNTR0	((volatile word*)(ISPBAR + 0x150004)) /* PIT count register  */

#define PIT0_INTVECTOR 55







#define QSMCR	((volatile word*) 0xfffc00)	/* QSM module configuration register  */
#define QSMIR	((volatile word*) 0xfffc04)	/* QSM interrupt register             */
#define QILR	((volatile word*) 0xfffc04)	/* QSM interrupt level register       */
#define QIVR	((volatile word*) 0xfffc05)	/* QSM interrupt vector register      */
#define SCCR0	((volatile word*) 0xfffc08)	/* SCI control register 0             */
#define SCCR1	((volatile word*) 0xfffc0a)	/* SCI control register 1             */
#define SCSR	((volatile word*) 0xfffc0c)	/* SCI status register                */
#define SCDR	((volatile word*) 0xfffc0e)	/* SCI data register                  */

/* exception vector table */

#define EVB 	((void**) 0x000000)	/* exception vector base */

/* register masks */

/* SYNCR bit masks */
#define SYN_W	0x8000	/* frequency control   */
#define SYN_X	0x4000	/* frequency prescale  */
#define SYN_Y	0x3f00	/* modulus             */

/* QSMCR bit masks */
#define STOP	0x8000	/* stop enable         */
#define FRZ1	0x4000	/* freeze control      */
#define FRZ0	0x2000
#define SUPV	0x0080	/* supervisor/unrestricted */
#define IARB	0x000f	/* interrupt arbitration   */

/* QSMIR bit masks */
#define ILQSPI	0x3800	/* interrupt level for QSPI             */
#define ILSCI	0x0700	/* interrupt level for SCI              */
#define INTV	0x00ff	/* QSPI/SCI interrupt vector number     */

/* SCCR bit masks */
#define SCBR	0x1fff	/* SCI baud rate                        */
#define LOOPS	0x4000	/* loop mode                            */
#define WOMS	0x2000	/* wired-or mode for SCI pins           */
#define ILT	0x1000	/* idle-line detect type                */
#define PT	0x0800	/* parity type                          */
#define PE	0x0400	/* parity enable                        */
#define M	0x0200	/* mode select (10/11-bit frame)        */
#define WAKE	0x0100	/* wakeup by address mark               */
#define TIE	0x0080	/* transmit interrupt enable            */
#define TCIE	0x0040	/* transmit complete interrupt enable   */
#define RIE	0x0020	/* receiver interrupt enable            */
#define ILIE	0x0010	/* idle-line interrupt enable           */
#define TE	0x0008	/* transmitter enable                   */
#define RE	0x0004	/* receiver enable                      */
#define RWU	0x0002	/* receiver wakeup                      */
#define SBK	0x0001	/* send break                           */

/* SCSR bit masks */
#define TDRE	0x0100	/* transmit data receiver empty         */
#define TC	0x0080	/* transmit complete                    */
#define RDRF	0x0040	/* receive data register full           */
#define RAF	0x0020	/* receiver active                      */
#define IDLE	0x0010	/* idle-line detected                   */
#define OR	0x0008	/* overrun error                        */
#define NF	0x0004	/* noise error flag                     */
#define FE	0x0002	/* framing error                        */
#define PF	0x0001	/* parity error                         */

/* interrupt levels */

#define DEF_INT_MASK	0xf0ff	/* default interrupt mask              */
#define QSM_INTLEVEL	0x1b00	/* QSPI and SCI interrupt level (3/3)  */
#define QSM_IARB	0x0111	/* QSM interrupt arbitration value     */

/* exception vectors */

#define INITIAL_SP	0x00	/* initial supervisor stack pointer contents */
#define INITIAL_PC	0x01	/* initial program counter contents          */
#define QSM_INTVECTOR	0xfe	/* QSPI and SCI interrupt vector number      */

/* other constants */

#define QUEUE_SIZE	64

#endif	/* _MCF5282_H_ */
